	/*
    bttv - Bt848 frame grabber driver

    Copyright (C) 1996,97 Ralph Metzler (rjkm@thp.uni-koeln.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _BTTVP_H_
#define _BTTVP_H_

#define CP(x) if(0)printk("BTTV CHECKPOINT " __FILE__ " %d %d " x "\n", __LINE__, btv->id)

#define BTTV_VERSION_CODE KERNEL_VERSION(0,7,48)

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0) 
# include "kcompat24.h"
# define vmalloc_32 vmalloc
# define virt_to_page MAP_NR
#endif

#include <linux/types.h>
#include <linux/wait.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>

#include "bt848.h"
#include "bttv.h"
#include "audiochip.h"

#ifdef __KERNEL__

/* ---------------------------------------------------------- */
/* bttv-driver.c                                              */

/* insmod options */
extern unsigned int bttv_verbose;
extern unsigned int bttv_debug;
extern unsigned int bttv_gpio;
extern void bttv_gpio_tracking(struct bttv *btv, char *comment);

/* Anybody who uses more than four? */
#define BTTV_MAX 4
extern int bttv_num;			/* number of Bt848s in use */
extern struct bttv bttvs[BTTV_MAX];


#ifndef O_NONCAP  
#define O_NONCAP	O_TRUNC
#endif

struct bttv_pll_info {
	unsigned int pll_ifreq;	   /* PLL input frequency 	 */
	unsigned int pll_ofreq;	   /* PLL output frequency       */
	unsigned int pll_crystal;  /* Crystal used for input     */
	unsigned int pll_current;  /* Currently programmed ofreq */
};

/* OPEN STRUCTURES */

#define MAX_OPENS 4
#define MAX_VBI_OPENS 4

/* GEOMETRY STRUCTURES */

struct geo_registers
{
	int width, height, v4l2format;
	u16 vtc;
	u16 hscale_lo;
	u16 hscale_hi;
	u16 vscale_lo;
	u16 vscale_hi;
	u16 hactive;
	u16 hdelay;
	u16 vactive;
	u16 vdelay;
	u16 crop;
	u16 format;
	u16 bswap;
	u16 uvswap;
	u16 comb;
};

/* hardware buffers */

/*  Bus-master scatter list  */
struct scatter_node
{
	__u32   addr;
	__u32   len;
};
#define END_OF_SCATTER_LIST     0x80000000 

struct hw_buf
{
	char *data; //vmalloc() data
	struct scatter_node *dma;
	u32 *risc_even;
	u32 *risc_odd;
	unsigned long risc_even_bus;
	unsigned long risc_odd_bus;
	int vma_nopen;
	int risc_len;
};

struct stream_buffer
{
	struct v4l2_q_node q;
	struct v4l2_buffer buf;
	struct hw_buf hw;
	struct bttvc *c;
	struct stream_buffer *next;
};

struct bttvc
{
	int enable;		/* this capture is active */
	int req;		/* this open has stream buffers requested */
	int map;			/* capture buffers mapped */
	int cap_contig;			/* buffers mapped contiguously */
	__s64 start_time;	/* stream start time */
	__s64 int_time;		/* time of last interrupt */
	__s64 frame_no;		/* number of this frame */
	__s64 last_frame;	/* number of last frame  */
	struct stream_buffer *bufs;	/* buffers */
	struct v4l2_queue cap_wait_q;	/* buffers ready for data */
	struct v4l2_queue cap_done_q;	/* buffers aleady done */
	struct stream_buffer *wait_odd;	/* buffer waiting for odd field */
	struct stream_buffer *wait_even;/* buffer wiating for even field */
	int ref_even;		/* even field referenced */
	int ref_odd;		/* odd field referenced */
	struct bttvc *next;		/* linked list of streaming opens */
	struct bttvo *open;		/* the open we're linked to */ 
};

struct bttvo
{
	int isopen;		/* this open is in use */
	int vbi;
	int index;
	int noncapturing;	/* this is a noncapturing open */
	int input;		/* the input for this open */
	int reading;		/* this open has read() resources */
	struct v4l2_buffer read_buffer;
	struct v4l2_performance perf;

	int bsize;			/* capture buffer size (per image) */
	struct v4l2_pix_format cap_fmt; /* capture format */
	struct geo_registers cap_geo;	/* geometry */
	struct hw_buf cap_dummy;	/* dummy buffer */

	struct v4l2_vbi_format vbi_fmt;

	struct bttvc read;
	struct bttvc stream;

	wait_queue_head_t waitq;		/* waitq for completed capture */

	struct bttv *dev;	/* link back to host device */
};

struct bttv {
	struct v4l2_device video_dev;
	struct v4l2_device radio_dev;
	struct v4l2_device vbi_dev;
	spinlock_t s_lock;
	struct semaphore lock;

	/* opens */
	int open_count;
	struct bttvo open_data[MAX_OPENS];
	struct bttvc *streaming_caps;

	int vbi_opens;
	struct bttvo vbi_open_data[MAX_VBI_OPENS];
	struct bttvc *streaming_vbis;
	int vbi_bpl;

	/* video standard */
	int norm;
	int input;
	uint freq;
	struct v4l2_pix_format cap_fmt;
	struct v4l2_vbi_format vbi_fmt;

	/* v4l2 stuff */
	unsigned long tpf;
	int last_irq;
	int combfilter;
	int automute;
	int vcrhack;

	/* Dummy buffers */
	struct hw_buf dummy_buf;
	int dummy_width;
	int dummy_height;
	int dummy_pfmt;
	struct geo_registers dummy_geo;

	/* DMA */
	int dma;
	int sync;
	int pvstat;
	int e_stat;
	int o_stat;
	int pv;

	/* Preview resources - only one set */
	struct hw_buf pv_buf;
	struct geo_registers pv_geo;
	struct v4l2_framebuffer fb;
	struct v4l2_window win;
	u32 bmsize;
	struct bttvo *pv_open;

	wait_queue_head_t riscq;

#define STAT_DUMMY -1
#define STAT_PV -3

#define PV_INIT 0
#define PV_FBUF 1
#define PV_WIN 2
#define PV_OVL 3

	u32 * risc;
	u32 * vbi_dummy;
	u32 * cap_odd_jmp;
	u32 * cap_even_jmp;
	u32 * vbi_odd_jmp;
	u32 * vbi_even_jmp;
	u32 * vbi_odd_tab;
	u32 * vbi_even_tab;
	u32 cap_odd_end;
	u32 cap_even_end;
	u32 vbi_dummy_phys;
	u32 vbi_odd_end;
	u32 vbi_even_end;
	u32 vbi_odd_start;
	u32 vbi_even_start;

	/* i2c */
	struct i2c_adapter i2c_adap;
	struct i2c_algo_bit_data   i2c_algo;
	struct i2c_client  i2c_client;
	int i2c_state, i2c_rc;
	struct i2c_client *i2c_clients[I2C_CLIENTS_MAX];

	int tuner_type;

	unsigned int nr;
	unsigned short id;
	struct pci_dev *dev;
	unsigned int irq;  /* IRQ used by Bt848 card */
	unsigned char revision;
	unsigned long bt848_adr;      /* bus address of IO mem returned by PCI BIOS */
	unsigned char *bt848_mem;   /* pointer to mapped IO memory */

 	int type;    /* card type  */
	int cardid;
	int audio;   /* audio mode */
	int audio_chip;      /* set to one of the chips supported by bttv.c */
	int radio;

	struct bttv_pll_info pll;
	int i2c_command;
	int triton1;

	int errors;
	int needs_restart;

	wait_queue_head_t gpioq;
	int shutdown;
};
#endif

#if defined(__powerpc__) /* big-endian */
extern __inline__ void io_st_le32(volatile unsigned *addr, unsigned val)
{
__asm__ __volatile__ ("stwbrx %1,0,%2" : \
    "=m" (*addr) : "r" (val), "r" (addr));
      __asm__ __volatile__ ("eieio" : : : "memory");
}

#define btwrite(dat,adr)  io_st_le32((unsigned *)(btv->bt848_mem+(adr)),(dat))
#define btread(adr)       ld_le32((unsigned *)(btv->bt848_mem+(adr)))
#else
#define btwrite(dat,adr)    writel((dat), (char *) (btv->bt848_mem+(adr)))
#define btread(adr) readl(btv->bt848_mem+(adr))
#endif

#define btand(dat,adr)      btwrite((dat) & btread(adr), adr)
#define btor(dat,adr)       btwrite((dat) | btread(adr), adr)
#define btaor(dat,mask,adr) btwrite((dat) | ((mask) & btread(adr)), adr)

/* bttv ioctls */

#define BTTV_READEE		_IOW('v',  BASE_VIDIOCPRIVATE+0, char [256])
#define BTTV_WRITEE		_IOR('v',  BASE_VIDIOCPRIVATE+1, char [256])
#define BTTV_FIELDNR		_IOR('v' , BASE_VIDIOCPRIVATE+2, unsigned int)
#define BTTV_PLLSET		_IOW('v' , BASE_VIDIOCPRIVATE+3, struct bttv_pll_info)
#define BTTV_BURST_ON      	_IOR('v' , BASE_VIDIOCPRIVATE+4, int)
#define BTTV_BURST_OFF     	_IOR('v' , BASE_VIDIOCPRIVATE+5, int)
#define BTTV_VERSION  	_IOR('v' , BASE_VIDIOCPRIVATE+6, int)
#define BTTV_PICNR		_IOR('v' , BASE_VIDIOCPRIVATE+7, int)
#define BTTV_VBISIZE    _IOR('v' , BASE_VIDIOCPRIVATE+8, int)

#define TDA9850    0x01
#define TDA9840    0x02
#define TDA8425    0x03
#define TEA6300    0x04

#endif

/*
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
